/*****************************************************
 * A class representing an email message.  The class
 * maintains all of the information necessary for an
 * email message (sender, recipients, subject, body,
 * attachments) and can convert all of this information
 * into a string that can be fed directly to an SMTP
 * server to produce a valid email.
 */

#ifndef EmailMessage_Included
#define EmailMessage_Included

#include <vector>
#include <string>
using namespace std;

class EmailMessage
{
public:
	/* string getSender()
	 * Usage: string from = message.getSender();
	 * ----------------------------------------------------
	 * Returns a string representation of the sender of the
	 * message in an SMTP-compliant format.  If the sender
	 * has not yet been specified, this function returns the
	 * empty string.
	 */
	string getSender();

	/* void setSender(string from)
	 * Usage: message.setSender("htiek@cs.stanford.edu");
	 * ----------------------------------------------------
	 * Sets the email message's sender to the specified
	 * email address.
	 */
	void setSender(string from);

	/* Type: RecipientType
	 * -------------------------------------------------
	 * A type representing what sort of recipient someone
	 * is.
	 */
	enum RecipientType {To, Cc, Bcc};

	/* void addRecipient(string to, RecipientType)
	 * Usage: message.addRecipient("htiek@cs.stanford.edu", EmailMessage::To);
	 * ----------------------------------------------------
	 * Adds a new recipient to the list of recipients for
	 * this email message.
	 */
	void addRecipient(string to, RecipientType mode);

	/* size_t numRecipients()
	 * Usage: for (size_t i = 0; i < message.numRecipients; ++i)
	 * ----------------------------------------------------
	 * Returns how many recipients this message has.
	 */
	size_t numRecipients();

	/* string getRecipient(size_t index)
	 * Usage: string firstRecipient = message.getRecipient(0);
	 * -------------------------------------------------
	 * Returns the email address at the specified position
	 * in the recipients list.  If the index is out of bounds,
	 * this function has undefined behavior.
	 */
	string getRecipient(size_t index);

	/* void setSubject()
	 * Usage: message.setSubject("Take my survey plz!!");
	 * ----------------------------------------------------
	 * Sets the message subject.
	 */
	void setSubject(string subject);

	/* void setMessageText(string text)
	 * Usage: message.setMessageText(text);
	 * -------------------------------------------------
	 * Sets the contents of the body of the message to the
	 * specified text.  Lines will be wrapped to no more than
	 * 64 characters.
	 */
	void setMessageText(string text);

	/* bool addAttachment(string filename);
	 * Usage: if (message.addAttachment("main.cpp"))
	 * ----------------------------------------------------
	 * Adds the specified file as an attachment to the email.
	 * If the file does not exist or this function has trouble
	 * adding it as an attachment, addAttachment returns false.
	 * Otherwise, the function returns true.
	 */
	bool addAttachment(string filename);

	/* string toString();
	 * Usage: cout << message.toString() << endl;
	 * -------------------------------------------------
	 * Produces a version of the message that can be sent
	 * via SMTP that contains the message information, along
	 * with all of the attachments.  To complete the
	 * assignment, you will need to transmit this data to
	 * Stanford's mail server.
	 */
	string toString();

	/* void clear();
	 * Usage: message.clear();
	 * -------------------------------------------------
	 * Resets the message, removing recipients, attachments,
	 * clearing the subject and sender, and erasing the
	 * message text.
	 */
	void clear();

private:

	/* Attachments store the base64-encoded data and the mime type. */
	struct Attachment
	{
		string filename;
		string data;
		string mimeType;
	};

	string                               from;
	vector<pair<string, RecipientType> > to;
	string                               subject;
	string                               message;
	vector<Attachment>                   attachments;
};


#endif